# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "ExperimentMetricDirectionEnum",
    },
)


class ExperimentMetricDirectionEnum(proto.Message):
    r"""Container for enum describing the type of experiment metric
    direction.

    """

    class ExperimentMetricDirection(proto.Enum):
        r"""The type of experiment metric direction.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            NO_CHANGE (2):
                The goal of the experiment is to not change
                the metric.
            INCREASE (3):
                The goal of the experiment is to increate the
                metric.
            DECREASE (4):
                The goal of the experiment is to decrease the
                metric.
            NO_CHANGE_OR_INCREASE (5):
                The goal of the experiment is to either not
                change or increase the metric.
            NO_CHANGE_OR_DECREASE (6):
                The goal of the experiment is to either not
                change or decrease the metric.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NO_CHANGE = 2
        INCREASE = 3
        DECREASE = 4
        NO_CHANGE_OR_INCREASE = 5
        NO_CHANGE_OR_DECREASE = 6


__all__ = tuple(sorted(__protobuf__.manifest))
